--[[
Name: uiQuery 1.0
Revision: $Rev: 25921 $
Website: http://www.wowace.com/
Description: ui wrapper
Dependencies: AceLibrary
]]

local MAJOR_VERSION = "uiQuery-1.0"
local MINOR_VERSION = "$Revision: 25921 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local prototypes = {}

prototypes["Frame"] = {
    size = function(self, h, w, s) 
        local frame = self.__frame
        if not h and not w and not s then
            return frame:GetHeight(), frame:GetWidth(), frame:GetScale()
        else
            if h then frame:SetHeight(h) end
            if w then frame:SetWidth(w) end
            if s then frame:SetScale(s) end
        end
        
        return self
    end,
}

prototypes["Region"] = {
    center = function(self, relTo, offX, offY)
        self.__frame:SetPoint("CENTER", relTo, "CENTER", offX, offY)
        return self
    end,
}

local function __handler(self, ...)
    if select('#', ...) > 0 then return ... else return self end
end

local function __index(self, key)
    local key = key
    if self.__frame[key] then
        self[key] = function(self, ...) return __handler(self, self.__frame[key](self.__frame, ...)) end
        return self[key]
    else
        return self
    end
end


local uiQuery = {}

function uiQuery:__call(frame)
    
    if not self.registry[frame] then
        local query = { __frame = frame, __index = __index}
        
        
        for key, value in pairs(prototypes) do
            if frame:IsObjectType(key) then
                for k2, v2 in pairs(value) do
                    if k2 ~= "events" and not query[k2] then query[k2] = v2 end                
                end
            end
        end
        
        setmetatable(query, query)
        self.registry[frame] = query
    end
    
    return self.registry[frame]
end

local function activate(self, oldLib, oldDeactivate)
    uiQuery = self
    
    if oldLib then
        self.registry = oldLib.registry
    else
        self.registry = {}
    end
    
    setmetatable(uiQuery, uiQuery)
    
    if oldDeactivate then
        oldDeactivate(oldLib)
    end
end

AceLibrary:Register(uiQuery, MAJOR_VERSION, MINOR_VERSION, activate)
uiQuery = AceLibrary(MAJOR_VERSION)